//-----------------------------------------------------------
//
//-----------------------------------------------------------
class ChosenPawn extends xPawn;
var bool isChosen;
var int chosenLevel;
var bool isFrozen;
var int chosenHealth;
//var string oldState;
var Material initialHead, initialBody, ChosenHead, ChosenBody;

function ClientRestart()
{
  isChosen = false;
  chosenLevel = 0;
  chosenHealth = 100;
  initialHead = Skins[0];
  initialBody = Skins[1];
  super.ClientRestart();
}

function PostBeginPlay()
{
  isChosen = false;
  chosenLevel = 0;
  chosenHealth = 100;
  initialHead = skins[0];
  initialBody = skins[1];
  super.PostBeginPlay();
}

/*
* IncreaseChosenLevel()
* This function will increase the chosen level of the player.
*/
function IncreaseChosenLevel()
{
   local Inventory tempChosenMoney;
   // have to go through the pawns inventory
   tempChosenMoney = FindInventoryType(class'ChosenMoney');
   if(tempChosenMoney != none)
   {
     chosenLevel = ChosenMoney(tempChosenMoney).ChosenMoneyAmount;
     ChosenPlayerReplicationInfo(self.Controller.PlayerReplicationInfo).chosenLevel = self.chosenLevel;
   }
}

/*
* Have to override Died to broadcast special messages, reset the chosen information
* and set the skins back to default if necessary
*/
function Died(Controller Killer, class<DamageType> damageType, vector HitLocation)
{
  local ChosenMoney myChosenMoney;
  local ChosenPawn frozenPawn;

  myChosenMoney = Spawn(Class'Chosen.ChosenMoney',,, HitLocation,);
  myChosenMoney.DropMoney(Location, chosenLevel);

  // if the pawn was chosen, broadcast the chosen killed message and
  // unfreeze any pawns
  if(isChosen)
  {
    BroadcastLocalizedMessage(class'Chosen.ChosenMessage',2,Killer.PlayerReplicationInfo,,);
    // unfreeze all frozen pawns
    ChosenMatch(Level.Game).theChosenOne = none;
    foreach allactors(class'ChosenPawn', frozenPawn)
    {
      frozenPawn.isFrozen = false;
      frozenPawn.chosenHealth = 100;
    }
  }

  // reset the chosen level and groundspeed
  chosenLevel = 0;
  GroundSpeed = 440.00;
  ChosenPlayerReplicationInfo(Controller.PlayerReplicationInfo).chosenLevel = 0;
  isChosen = false;
  ChosenPlayerReplicationInfo(Controller.PlayerReplicationInfo).isChosen = false;

  // set back to old skins
  Skins[0] = initialHead;
  Skins[1] = initialBody;

  super.Died(Killer,damageType,HitLocation);
}

/*
* CheckChosen()
* If the chosenLevel >= 5, return true
*/
function bool CheckChosen()
{
  if(chosenLevel >= 5)
    return true;
  return false;
}

/*
* SetChosen()
* Set chosen will set the pawn up as being chosen.
*/
function SetChosen()
{
  isChosen = true;
  self.GiveWeapon("Chosen.ChosenWeapon");
  self.SwitchWeapon(15);
  // when we are here, we are chosen.  Make the pawn kick some ass
  Health = 150;
  GroundSpeed = 660;
  // change the skins
  initialHead = Skins[0];
  initialBody = Skins[1];
  Skins[0]= ChosenHead;
  Skins[1]= ChosenBody;
  // set all enemy references to you
  setChosenAsEnemy();
  // tell the game you are the chosen one
  ChosenMatch(Level.Game).theChosenOne = self;
  // set up player rep, and broadcast the chosen message
  ChosenPlayerReplicationInfo(Controller.PlayerReplicationInfo).isChosen = true;
  BroadcastLocalizedMessage(class'Chosen.ChosenMessage',1,Controller.PlayerReplicationInfo,,);


}

// we want all bots to think of you as the enemy
function setChosenAsEnemy()
{
  local ChosenBot tempBot;

  foreach allactors(class'ChosenBot', tempBot)
  {
    if((ChosenPawn(tempBot.Pawn) != none)&&(ChosenPawn(tempBot.Pawn) != self))
      tempBot.Enemy = self;
  }
}



/*
* have to override TakeDamage so that if the pawn is taking chosen damage
* it will be reflected here
*/
function TakeDamage( int Damage, Pawn instigatedBy, Vector hitlocation,
							vector momentum, class<DamageType> damageType)
{
  if(damageType == class'Chosen.ChosenDamage')
  {
    chosenHealth -= Damage;
    if(chosenHealth <= 0)
    {
      // have to determine if player or bot
      if(self.IsHumanControlled())
        ChosenPlayer(Controller).Freeze();
      else
        ChosenBot(Controller).Freeze();
    }
  }
  else
  {
    if(!isFrozen)
      super.TakeDamage(Damage, instigatedBy, hitlocation, momentum, damageType);
  }
}




DefaultProperties
{
  ControllerClass=Class'Chosen.ChosenBot'
  ChosenHead = Texture'Aw-2004Particles.AirBlastP'
  ChosenBody = Texture'Aw-2004Particles.AirBlastP'
}
