//-----------------------------------------------------------
//
//-----------------------------------------------------------
class ChosenPlayer extends xPlayer;

/**
 * Adjusts aim for the third person perspective
 */
function rotator AdjustAim(FireProperties FiredAmmunition, vector projStart, int aimerror)
{
    local vector FireDir, AimSpot, HitNormal, HitLocation, OldAim, AimOffset;
    local actor BestTarget;
    local float bestAim, bestDist, projspeed;
    local actor HitActor;
    local bool bNoZAdjust, bLeading;
    local rotator AimRot;

    FireDir = vector(Rotation);
    if ( FiredAmmunition.bInstantHit )
        HitActor = Trace(HitLocation, HitNormal, projStart + 10000 * FireDir, projStart, true);
    else
        HitActor = Trace(HitLocation, HitNormal, projStart + 4000 * FireDir, projStart, true);
    if ( (HitActor != None) && HitActor.bProjTarget )
    {
        BestTarget = HitActor;
        bNoZAdjust = true;
        OldAim = HitLocation;
        BestDist = VSize(BestTarget.Location - Pawn.Location);
    }
    else
    {
        // adjust aim based on FOV
        bestAim = 0.90;
        if ( bAimingHelp )
        {
            bestAim = 0.93;
            if ( FiredAmmunition.bInstantHit )
                bestAim = 0.97;
            if ( FOVAngle < DefaultFOV - 8 )
                bestAim = 0.99;
        }
        else if ( FiredAmmunition.bInstantHit )
                bestAim = 1.0;
        BestTarget = PickTarget(bestAim, bestDist, FireDir, projStart, FiredAmmunition.MaxRange); //amb
        if ( BestTarget == None )
				return Rotation;
        OldAim = projStart + FireDir * bestDist;
    }
	//FiredAmmunition.WarnTarget(BestTarget,Pawn,FireDir);
    if ( (!bAimingHelp) || (Level.NetMode != NM_Standalone) )
            return Rotation;

    // aim at target - help with leading also
    if ( !FiredAmmunition.bInstantHit )
    {
        projspeed = FiredAmmunition.ProjectileClass.default.speed;
        BestDist = vsize(BestTarget.Location + BestTarget.Velocity * FMin(2, 0.02 + BestDist/projSpeed) - projStart);
        bLeading = true;
        FireDir = BestTarget.Location + BestTarget.Velocity * FMin(2, 0.02 + BestDist/projSpeed) - projStart;
        AimSpot = projStart + bestDist * Normal(FireDir);
        // if splash damage weapon, try aiming at feet - trace down to find floor
        if ( FiredAmmunition.bTrySplash
            && ((BestTarget.Velocity != vect(0,0,0)) || (BestDist > 1500)) )
        {
            HitActor = Trace(HitLocation, HitNormal, AimSpot - BestTarget.CollisionHeight * vect(0,0,2), AimSpot, false);
            if ( (HitActor != None)
                && FastTrace(HitLocation + vect(0,0,4),projstart) )
                return rotator(HitLocation + vect(0,0,6) - projStart);
        }
    }
    else
    {
        FireDir = BestTarget.Location - projStart;
        AimSpot = projStart + bestDist * Normal(FireDir);
    }
    AimOffset = AimSpot - OldAim;

    // adjust Z of shooter if necessary
    if ( bNoZAdjust || (bLeading && (Abs(AimOffset.Z) < BestTarget.CollisionHeight)) )
        AimSpot.Z = OldAim.Z;
    else if ( AimOffset.Z < 0 )
        AimSpot.Z = BestTarget.Location.Z + 0.4 * BestTarget.CollisionHeight;
    else
        AimSpot.Z = BestTarget.Location.Z - 0.7 * BestTarget.CollisionHeight;

    if ( !bLeading )
    {
        // if not leading, add slight random error ( significant at long distances )
        if ( !bNoZAdjust )
        {
            AimRot = rotator(AimSpot - projStart);
            if ( FOVAngle < DefaultFOV - 8 )
                AimRot.Yaw = AimRot.Yaw + 200 - Rand(400);
            else
                AimRot.Yaw = AimRot.Yaw + 375 - Rand(750);
            return AimRot;
        }
    }
    else if ( !FastTrace(projStart + 0.9 * bestDist * Normal(FireDir), projStart) )
    {
        FireDir = BestTarget.Location - projStart;
        AimSpot = projStart + bestDist * Normal(FireDir);
    }

    return rotator(AimSpot - projStart);
}

// possess the pawn
function Possess( Pawn aPawn )
{
    local ChosenPawn vp;

    Super.Possess( aPawn );

    vp = ChosenPawn( aPawn );
	if ( vp != none )
		vp.Setup( PawnSetupRecord, true );
}


event PlayerCalcView(out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
    local vector dist;
    local float chaseDistance;

    chaseDistance = 150;
    dist = chaseDistance * vector(CameraRotation);

    bBehindView = true;
    ViewActor = ViewTarget;
    CameraLocation = ViewActor.Location;
    CameraLocation = CameraLocation - dist;
    CameraLocation.Z += 50;
}

function Freeze()
{
    ChosenPawn(Pawn).chosenHealth = -100;
    GoToState('Frozen');
}
// the frozen state will be where players wait after they are killed by the chosen one.
state Frozen
{     ignores SeePlayer, EnemyNotVisible, HearNoise, ReceiveWarning, NotifyLanded, NotifyPhysicsVolumeChange,
		NotifyTakeHit,NotifyHeadVolumeChange,NotifyLanded,NotifyHitWall,NotifyBump;

    exec function Fire( optional float F )
    {
    }

    exec function AltFire( optional float F )
    {
    }

   function BeginState()
   {
      ChosenPawn(Pawn).isFrozen = true;
   }

   function EndState()
   {
      ChosenPawn(Pawn).chosenHealth = 100;
   }

   Begin:
     if(ChosenPawn(Pawn).isFrozen)
     {
       ChosenPawn(Pawn).SetPhysics(PHYS_None);
       ChosenPawn(Pawn).PlayVictoryAnimation();
       ChosenPawn(Pawn).StopWeaponFiring();
     }
     else
     {
       ChosenPawn(Pawn).chosenHealth = 100;
       GotoState('PlayerWalking');
     }
     Sleep(0.5);
     GotoState('Frozen', 'Begin');
}


defaultproperties
{
     bBehindView=True
     PlayerReplicationInfoClass=Class'Chosen.ChosenPlayerReplicationInfo'
     PawnClass=Class'Chosen.ChosenPawn'
}
